@@ -66,7 +66,7 @@ func FinalizeCephCommandArgs(command string, clusterInfo *ClusterInfo, args []st
 
 	// we could use a slice and iterate over it but since we have only 3 elements
 	// I don't think this is worth a loop
-	timeout := strconv.Itoa(int(exec.CephCommandTimeout.Seconds()))
+	timeout := strconv.Itoa(int(exec.CephCommandsTimeout.Seconds()))
 	if command != "rbd" && command != "crushtool" && command != "radosgw-admin" {
 		args = append(args, "--connect-timeout="+timeout)
 	}
@@ -96,7 +96,6 @@ type CephToolCommand struct {
 	args            []string
 	timeout         time.Duration
 	JsonOutput      bool
-	OutputFile      bool
 	RemoteExecution bool
 }
 
@@ -107,7 +106,6 @@ func newCephToolCommand(tool string, context *clusterd.Context, clusterInfo *Clu
 		clusterInfo: clusterInfo,
 		args:        args,
 		JsonOutput:  true,
-		OutputFile:  true,
 	}
 }
 
@@ -118,7 +116,6 @@ func NewCephCommand(context *clusterd.Context, clusterInfo *ClusterInfo, args []
 func NewRBDCommand(context *clusterd.Context, clusterInfo *ClusterInfo, args []string) *CephToolCommand {
 	cmd := newCephToolCommand(RBDTool, context, clusterInfo, args)
 	cmd.JsonOutput = false
-	cmd.OutputFile = false
 
 	// When Multus is enabled, the RBD tool should run inside the proxy container
 	if clusterInfo.NetworkSpec.IsMultus() {
@@ -129,7 +126,22 @@ func NewRBDCommand(context *clusterd.Context, clusterInfo *ClusterInfo, args []s
 }
 
 func (c *CephToolCommand) run() ([]byte, error) {
-	command, args := FinalizeCephCommandArgs(c.tool, c.clusterInfo, c.args, c.context.ConfigDir)
+	// Initialize the command and args
+	command := c.tool
+	args := c.args
+
+	// If this is a remote execution, we don't want to build the full set of args. For instance all
+	// these args are not needed since those paths don't exist inside the cmd-proxy container:
+	//      --cluster=openshift-storage
+	//		--conf=/var/lib/rook/openshift-storage/openshift-storage.config
+	//		--name=client.admin
+	//		--keyring=/var/lib/rook/openshift-storage/client.admin.keyring
+	//
+	// The cmd-proxy container will take care of the rest with the help of the env CEPH_ARGS
+	if !c.RemoteExecution {
+		command, args = FinalizeCephCommandArgs(c.tool, c.clusterInfo, c.args, c.context.ConfigDir)
+	}
+
 	if c.JsonOutput {
 		args = append(args, "--format", "json")
 	} else {
@@ -142,40 +154,23 @@ func (c *CephToolCommand) run() ([]byte, error) {
 	var output, stderr string
 	var err error
 
-	if c.OutputFile {
-		if command == Kubectl {
-			// Kubectl commands targeting the toolbox container generate a temp
-			// file in the wrong place, so we will instead capture the output
-			// from stdout for the tests
-			if c.timeout == 0 {
-				output, err = c.context.Executor.ExecuteCommandWithOutput(command, args...)
-			} else {
-				output, err = c.context.Executor.ExecuteCommandWithTimeout(c.timeout, command, args...)
-			}
-		} else {
-			if c.timeout == 0 {
-				output, err = c.context.Executor.ExecuteCommandWithOutputFile(command, "--out-file", args...)
-			} else {
-				output, err = c.context.Executor.ExecuteCommandWithOutputFileTimeout(c.timeout, command, "--out-file", args...)
-			}
-		}
-	} else {
-		// NewRBDCommand does not use the --out-file option so we only check for remote execution here
-		// Still forcing the check for the command if the behavior changes in the future
-		if command == RBDTool {
-			if c.RemoteExecution {
-				output, stderr, err = c.context.RemoteExecutor.ExecCommandInContainerWithFullOutputWithTimeout(ProxyAppLabel, CommandProxyInitContainerName, c.clusterInfo.Namespace, append([]string{command}, args...)...)
-				output = fmt.Sprintf("%s.%s", output, stderr)
-			} else if c.timeout == 0 {
-				output, err = c.context.Executor.ExecuteCommandWithOutput(command, args...)
-			} else {
-				output, err = c.context.Executor.ExecuteCommandWithTimeout(c.timeout, command, args...)
+	// NewRBDCommand does not use the --out-file option so we only check for remote execution here
+	// Still forcing the check for the command if the behavior changes in the future
+	if command == RBDTool {
+		if c.RemoteExecution {
+			output, stderr, err = c.context.RemoteExecutor.ExecCommandInContainerWithFullOutputWithTimeout(ProxyAppLabel, CommandProxyInitContainerName, c.clusterInfo.Namespace, append([]string{command}, args...)...)
+			if stderr != "" || err != nil {
+				err = errors.Errorf("%s. %s", err.Error(), stderr)
 			}
 		} else if c.timeout == 0 {
 			output, err = c.context.Executor.ExecuteCommandWithOutput(command, args...)
 		} else {
 			output, err = c.context.Executor.ExecuteCommandWithTimeout(c.timeout, command, args...)
 		}
+	} else if c.timeout == 0 {
+		output, err = c.context.Executor.ExecuteCommandWithOutput(command, args...)
+	} else {
+		output, err = c.context.Executor.ExecuteCommandWithTimeout(c.timeout, command, args...)
 	}
 
 	return []byte(output), err
@@ -196,7 +191,7 @@ func (c *CephToolCommand) RunWithTimeout(timeout time.Duration) ([]byte, error)
 // configured its arguments. It is future work to integrate this case into the
 // generalization.
 func ExecuteRBDCommandWithTimeout(context *clusterd.Context, args []string) (string, error) {
-	output, err := context.Executor.ExecuteCommandWithTimeout(exec.CephCommandTimeout, RBDTool, args...)
+	output, err := context.Executor.ExecuteCommandWithTimeout(exec.CephCommandsTimeout, RBDTool, args...)
 	return output, err
 }
 
