@@ -177,6 +177,12 @@ func (r *ReconcileCephFilesystem) reconcile(request reconcile.Request) (reconcil
 	if err != nil {
 		if kerrors.IsNotFound(err) {
 			logger.Debug("cephFilesystem resource not found. Ignoring since object must be deleted.")
+			// If there was a previous error or if a user removed this resource's finalizer, it's
+			// possible Rook didn't clean up the monitoring routine for this resource. Ensure the
+			// routine is stopped when we see the resource is gone.
+			cephFilesystem.Name = request.Name
+			cephFilesystem.Namespace = request.Namespace
+			r.cancelMirrorMonitoring(cephFilesystem)
 			return reconcile.Result{}, nil
 		}
 		// Error reading the object - requeue the request.
@@ -204,6 +210,9 @@ func (r *ReconcileCephFilesystem) reconcile(request reconcile.Request) (reconcil
 		// If not, we should wait for it to be ready
 		// This handles the case where the operator is not ready to accept Ceph command but the cluster exists
 		if !cephFilesystem.GetDeletionTimestamp().IsZero() && !cephClusterExists {
+			// don't leak the health checker routine if we are force deleting
+			r.cancelMirrorMonitoring(cephFilesystem)
+
 			// Remove finalizer
 			err := opcontroller.RemoveFinalizer(r.client, cephFilesystem)
 			if err != nil {
@@ -254,13 +263,7 @@ func (r *ReconcileCephFilesystem) reconcile(request reconcile.Request) (reconcil
 		}
 
 		// If the ceph fs still in the map, we must remove it during CR deletion
-		if fsChannelExists {
-			// Close the channel to stop the mirroring status
-			close(r.fsChannels[fsChannelKeyName(cephFilesystem)].stopChan)
-
-			// Remove ceph fs from the map
-			delete(r.fsChannels, fsChannelKeyName(cephFilesystem))
-		}
+		r.cancelMirrorMonitoring(cephFilesystem)
 
 		// Remove finalizer
 		err = opcontroller.RemoveFinalizer(r.client, cephFilesystem)
@@ -458,6 +461,18 @@ func (r *ReconcileCephFilesystem) reconcileAddBoostrapPeer(cephFilesystem *cephv
 	return nil
 }
 
-func fsChannelKeyName(cephFilesystem *cephv1.CephFilesystem) string {
-	return fmt.Sprintf("%s-%s", cephFilesystem.Namespace, cephFilesystem.Name)
+func fsChannelKeyName(f *cephv1.CephFilesystem) string {
+	return types.NamespacedName{Namespace: f.Namespace, Name: f.Name}.String()
+}
+
+// cancel mirror monitoring. This is a noop if monitoring is not running.
+func (r *ReconcileCephFilesystem) cancelMirrorMonitoring(cephFilesystem *cephv1.CephFilesystem) {
+	_, fsChannelExists := r.fsChannels[fsChannelKeyName(cephFilesystem)]
+	if fsChannelExists {
+		// Close the channel to stop the mirroring status
+		close(r.fsChannels[fsChannelKeyName(cephFilesystem)].stopChan)
+
+		// Remove ceph fs from the map
+		delete(r.fsChannels, fsChannelKeyName(cephFilesystem))
+	}
 }
